@@ -1,5 +1,6 @@
 # This file is part of MyPaint.
-# Copyright (C) 2014 by Andrew Chadwick <a.t.chadwick@gmail.com>
+# -*- coding: utf-8 -*-
+# Copyright (C) 2014-2019 by the MyPaint Development Team.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -11,8 +12,8 @@
 
 ## Imports
 
+from __future__ import division, print_function
 import logging
-logger = logging.getLogger(__name__)
 import collections
 import re
 
@@ -24,6 +25,7 @@ from gi.repository import Pango
 from lib.observable import event
 import gui.mode
 
+logger = logging.getLogger(__name__)
 
 ## Device prefs
 
@@ -239,9 +241,16 @@ class Monitor (object):
         """Gets the settings for a device
 
         :param Gdk.Device device: a physical ("slave") device
-        :returns: A settings object which can be manipulated
+        :returns: A settings object which can be manipulated, or None
         :rtype: Settings
 
+        Changes to the returned object made via its API are saved to the
+        user preferences immediately.
+
+        If the device is a keyboard, or is otherwise unsuitable as a
+        pointing device, None is returned instead. The caller needs to
+        check this case.
+
         """
         self._init_device_settings(device)
         return self._device_settings.get(device)
@@ -302,7 +311,7 @@ class Monitor (object):
         :returns: ultimately a sequence of (Gdk.Device, Settings) pairs
 
         """
-        for device, settings in self._device_settings.iteritems():
+        for device, settings in self._device_settings.items():
             yield (device, settings)
 
     ## Current device
@@ -345,7 +354,7 @@ class Monitor (object):
         logger.debug(
             "Device change: name=%r source=%s",
             new_device.name, new_device.source.value_name,
-            )
+        )
 
         # When editing brush settings, it is often more convenient to use the
         # mouse. Because of this, we don't restore brushsettings when switching
@@ -414,11 +423,11 @@ class SettingsEditor (Gtk.Grid):
         self._monitor = monitor
 
         self._devices_store = Gtk.ListStore(object)
-        self._devices_view = Gtk.TreeView(self._devices_store)
+        self._devices_view = Gtk.TreeView(model=self._devices_store)
 
-        #TRANSLATORS: Column's data is the device's name
         col = Gtk.TreeViewColumn(C_(
             "prefs: devices table: column header",
+            # TRANSLATORS: Column's data is the device's name
             "Device",
         ))
         col.set_min_width(200)
@@ -427,12 +436,12 @@ class SettingsEditor (Gtk.Grid):
         self._devices_view.append_column(col)
         cell = Gtk.CellRendererText()
         cell.set_property("ellipsize", Pango.EllipsizeMode.MIDDLE)
-        col.pack_start(cell, expand=True)
+        col.pack_start(cell, True)
         col.set_cell_data_func(cell, self._device_name_datafunc)
 
-        #TRANSLATORS: Column's data is an integer count of the number of axes
         col = Gtk.TreeViewColumn(C_(
             "prefs: devices table: column header",
+            # TRANSLATORS: Column's data is the number of axes (an integer)
             "Axes",
         ))
         col.set_min_width(30)
@@ -441,12 +450,12 @@ class SettingsEditor (Gtk.Grid):
         col.set_sizing(Gtk.TreeViewColumnSizing.AUTOSIZE)
         self._devices_view.append_column(col)
         cell = Gtk.CellRendererText()
-        col.pack_start(cell, expand=True)
+        col.pack_start(cell, True)
         col.set_cell_data_func(cell, self._device_axes_datafunc)
 
-        #TRANSLATORS: Column shows type labels ("Touchscreen", "Pen" etc.)
         col = Gtk.TreeViewColumn(C_(
             "prefs: devices table: column header",
+            # TRANSLATORS: Column shows type labels ("Touchscreen", "Pen" etc.)
             "Type",
         ))
         col.set_min_width(120)
@@ -456,7 +465,7 @@ class SettingsEditor (Gtk.Grid):
         self._devices_view.append_column(col)
         cell = Gtk.CellRendererText()
         cell.set_property("ellipsize", Pango.EllipsizeMode.END)
-        col.pack_start(cell, expand=True)
+        col.pack_start(cell, True)
         col.set_cell_data_func(cell, self._device_type_datafunc)
 
         # Usage config value => string store (dropdowns)
@@ -466,11 +475,11 @@ class SettingsEditor (Gtk.Grid):
             store.append([conf_val, string])
         self._usage_store = store
 
-        #TRANSLATORS: Column's data is a dropdown allowing the allowed
-        #TRANSLATORS: tasks for the row's device to be configured.
         col = Gtk.TreeViewColumn(C_(
             "prefs: devices table: column header",
-            "Use for...",
+            # TRANSLATORS: Column's data is a dropdown allowing the allowed
+            # TRANSLATORS: tasks for the row's device to be configured.
+            u"Use for…",
         ))
         col.set_min_width(100)
         col.set_resizable(True)
@@ -485,7 +494,7 @@ class SettingsEditor (Gtk.Grid):
         cell.set_property("has-entry", False)
         cell.set_property("ellipsize", Pango.EllipsizeMode.END)
         cell.connect("changed", self._usage_cell_changed_cb)
-        col.pack_start(cell, expand=True)
+        col.pack_start(cell, True)
         col.set_cell_data_func(cell, self._device_usage_datafunc)
 
         # Scroll action config value => string store (dropdowns)
@@ -495,12 +504,12 @@ class SettingsEditor (Gtk.Grid):
             store.append([conf_val, string])
         self._scroll_store = store
 
-        #TRANSLATORS: Column's data is a dropdown for how the device's
-        #TRANSLATORS: scroll wheel or scroll-gesture events are to be
-        #TRANSLATORS: interpreted normally.
         col = Gtk.TreeViewColumn(C_(
             "prefs: devices table: column header",
-            "Scroll...",
+            # TRANSLATORS: Column's data is a dropdown for how the device's
+            # TRANSLATORS: scroll wheel or scroll-gesture events are to be
+            # TRANSLATORS: interpreted normally.
+            u"Scroll…",
         ))
         col.set_min_width(100)
         col.set_resizable(True)
@@ -515,7 +524,7 @@ class SettingsEditor (Gtk.Grid):
         cell.set_property("has-entry", False)
         cell.set_property("ellipsize", Pango.EllipsizeMode.END)
         cell.connect("changed", self._scroll_cell_changed_cb)
-        col.pack_start(cell, expand=True)
+        col.pack_start(cell, True)
         col.set_cell_data_func(cell, self._device_scroll_datafunc)
 
         # Pretty borders
@@ -551,12 +560,16 @@ class SettingsEditor (Gtk.Grid):
     def _device_usage_datafunc(self, column, cell, model, iter_, *data):
         device = model.get_value(iter_, 0)
         settings = self._monitor.get_device_settings(device)
+        if not settings:
+            return
         text = AllowedUsage.DISPLAY_STRING[settings.usage]
         cell.set_property("text", text)
 
     def _device_scroll_datafunc(self, column, cell, model, iter_, *data):
         device = model.get_value(iter_, 0)
         settings = self._monitor.get_device_settings(device)
+        if not settings:
+            return
         text = ScrollAction.DISPLAY_STRING[settings.scroll]
         cell.set_property("text", text)
 
@@ -564,16 +577,20 @@ class SettingsEditor (Gtk.Grid):
 
     def _usage_cell_changed_cb(self, combo, device_path_str,
                                usage_iter, *etc):
-        config = self._usage_store.get_value(usage_iter, self._USAGE_CONFIG_COL)
+        config = self._usage_store.get_value(
+            usage_iter,
+            self._USAGE_CONFIG_COL,
+        )
         device_iter = self._devices_store.get_iter(device_path_str)
         device = self._devices_store.get_value(device_iter, 0)
         settings = self._monitor.get_device_settings(device)
+        if not settings:
+            return
         settings.usage = config
         self._devices_view.columns_autosize()
 
     def _scroll_cell_changed_cb(self, conf_combo, device_path_str,
                                 conf_iter, *etc):
-        mon = self._monitor
         conf_store = self._scroll_store
         conf_col = self._SCROLL_CONFIG_COL
         conf_value = conf_store.get_value(conf_iter, conf_col)
@@ -581,6 +598,8 @@ class SettingsEditor (Gtk.Grid):
         device_iter = device_store.get_iter(device_path_str)
         device = device_store.get_value(device_iter, 0)
         settings = self._monitor.get_device_settings(device)
+        if not settings:
+            return
         settings.scroll = conf_value
         self._devices_view.columns_autosize()
 
@@ -643,7 +662,7 @@ def _test():
     win.add(editor)
     win.show_all()
     Gtk.main()
-    print monitor._prefs
+    print(monitor._prefs)
 
 
 if __name__ == '__main__':
