@@ -1,5 +1,6 @@
 # This file is part of MyPaint.
 # Copyright (C) 2008-2013 by Martin Renold <martinxyz@gmx.ch>
+# Copyright (C) 2013-2018 by the MyPaint Development Team.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -9,37 +10,22 @@
 
 ## Imports
 
+from __future__ import division, print_function
 import math
-from numpy import array
-from numpy import isfinite
-from lib.helpers import clamp
 import logging
 from collections import deque
-logger = logging.getLogger(__name__)
-
-import gtk2compat
 from gettext import gettext as _
-import gobject
-import gtk
-from gtk import gdk
-from libmypaint import brushsettings
 
+from gi.repository import Gtk
+from gi.repository import Gdk
+from gi.repository import GLib
+import numpy as np
+
+from lib.helpers import clamp
 import gui.mode
-from drawutils import spline_4p
+from .drawutils import spline_4p
 
-from lib import mypaintlib
-
-
-## Module settings
-
-# Which workarounds to allow for motion event compression
-EVCOMPRESSION_WORKAROUND_ALLOW_DISABLE_VIA_API = True
-EVCOMPRESSION_WORKAROUND_ALLOW_EVHACK_FILTER = True
-
-# Consts for the style of workaround in use
-EVCOMPRESSION_WORKAROUND_DISABLE_VIA_API = 1
-EVCOMPRESSION_WORKAROUND_EVHACK_FILTER = 2
-EVCOMPRESSION_WORKAROUND_NONE = 999
+logger = logging.getLogger(__name__)
 
 
 ## Class defs
@@ -75,33 +61,13 @@ class FreehandMode (gui.mode.BrushworkModeMixin,
     # as the x and y coords, pressure and tilt prior to the strokes
     # rendering.
 
-    MOTION_QUEUE_PRIORITY = gobject.PRIORITY_DEFAULT_IDLE
+    MOTION_QUEUE_PRIORITY = GLib.PRIORITY_DEFAULT_IDLE
 
     # The Right Thing To Do generally is to spend as little time as
     # possible directly handling each event received. Disconnecting
     # stroke rendering from event processing buys the user the ability
     # to quit out of a slowly/laggily rendering stroke if desired.
 
-    # Due to later versions of GTK3 (3.8+) discarding successive motion
-    # events received in the same frame (so-called "motion
-    # compression"), we employ a GDK event filter (on platforms using
-    # Xi2) to capture coordinates faster than GDK deigns to pass them on
-    # to us. Reason: we want the fidelity that GDK refuses to give us
-    # currently for a fancy Wacom device delivering motion events at
-    # ~200Hz: frame clock speeds are only 50 or 60 Hz.
-
-    # https://gna.org/bugs/?21003
-    # https://gna.org/bugs/?20822
-    # https://bugzilla.gnome.org/show_bug.cgi?id=702392
-
-    # It's hard to capture and translate tilt and pressure info in a
-    # manner that's compatible with GDK without using GDK's own private
-    # internals.  Implementing our own valuator translations would be
-    # likely to break, so for these pices of info we interpolate the
-    # values received on the clock tick-synchronized motion events GDK
-    # gives us, but position them at the x and y that Xi2 gave us.
-    # Pressure and tilt fidelities matter less than positional accuracy.
-
     ## Initialization
 
     def __init__(self, ignore_modifiers=True, **args):
@@ -134,12 +100,6 @@ class FreehandMode (gui.mode.BrushworkModeMixin,
         def __init__(self):
             object.__init__(self)
 
-            # Ugly workarounds
-            self.event_compression_workaround = None
-            self.event_compression_was_enabled = None
-            self.evhack_data = None   # or (tdw, mode), identity matters!
-            self.evhack_positions = []
-
             self.last_event_had_pressure = False
 
             # Raw data which was delivered with an identical timestamp
@@ -159,14 +119,14 @@ class FreehandMode (gui.mode.BrushworkModeMixin,
 
             # Queued Event Handling
 
-            # Pressure and tilt interpolation for evhack events, which
-            # don't have pressure or tilt date.
+            # Pressure and tilt interpolation for events which
+            # don't have pressure or tilt data.
             self.interp = PressureAndTiltInterpolator()
 
             # Time of the last-processed event
             self.last_handled_event_time = 0
 
-            # Debugging: number of events procesed each second,
+            # Debugging: number of events processed each second,
             # average times.
             self.avgtime = None
 
@@ -178,6 +138,9 @@ class FreehandMode (gui.mode.BrushworkModeMixin,
             self.last_good_raw_pressure = 0.0
             self.last_good_raw_xtilt = 0.0
             self.last_good_raw_ytilt = 0.0
+            self.last_good_raw_viewzoom = 0.0
+            self.last_good_raw_viewrotation = 0.0
+            self.last_good_raw_barrel_rotation = 0.0
 
         def queue_motion(self, event_data):
             """Append one raw motion event to the motion queue
@@ -186,16 +149,21 @@ class FreehandMode (gui.mode.BrushworkModeMixin,
             :type event_data: tuple
 
             Events are tuples of the form ``(time, x, y, pressure,
-            xtilt, ytilt)``. Times are in milliseconds, and are
-            expressed as ints. ``x`` and ``y`` are ordinary Python
-            floats, and refer to model coordinates. The pressure and
-            tilt values have the meaning assigned to them by GDK; if
-            ```pressure`` is None, pressure and tilt values will be
-            interpolated from surrounding defined values.
+            xtilt, ytilt, viewzoom, viewrotation, barrel_rotation)``.
+            Times are in milliseconds, and are expressed as ints. ``x``
+            and ``y`` are ordinary Python floats, and refer to model
+            coordinates. The pressure and tilt values have the meaning
+            assigned to them by GDK; if ```pressure`` is None, pressure
+            and tilt values will be interpolated from surrounding
+            defined values.
 
             Zero-dtime events are detected and cleaned up here.
+
             """
-            time, x, y, pressure, xtilt, ytilt = event_data
+
+            (time, x, y, pressure,
+             xtilt, ytilt, viewzoom,
+             viewrotation, barrel_rotation) = event_data
             if time < self._last_queued_event_time:
                 logger.warning('Time is running backwards! Corrected.')
                 time = self._last_queued_event_time
@@ -204,7 +172,8 @@ class FreehandMode (gui.mode.BrushworkModeMixin,
                 # On Windows, GTK timestamps have a resolution around
                 # 15ms, but tablet events arrive every 8ms.
                 # https://gna.org/bugs/index.php?16569
-                zdata = (x, y, pressure, xtilt, ytilt)
+                zdata = (x, y, pressure, xtilt, ytilt, viewzoom,
+                         viewrotation, barrel_rotation)
                 self._zero_dtime_motions.append(zdata)
             else:
                 # Queue any previous events that had identical
@@ -218,11 +187,13 @@ class FreehandMode (gui.mode.BrushworkModeMixin,
                         interval = 100.0
                     else:
                         zt = self._last_queued_event_time
-                        interval = float(dtime)
+                        interval = dtime
                     step = interval / (len(self._zero_dtime_motions) + 1)
-                    for zx, zy, zp, zxt, zyt in self._zero_dtime_motions:
+
+                    for (zx, zy, zp, zxt,
+                         zyt, zvz, zvr, zbr) in self._zero_dtime_motions:
                         zt += step
-                        zevent_data = (zt, zx, zy, zp, zxt, zyt)
+                        zevent_data = (zt, zx, zy, zp, zxt, zyt, zvz, zvr, zbr)
                         self.motion_queue.append(zevent_data)
                     # Reset the backlog buffer
                     self._zero_dtime_motions = []
@@ -240,7 +211,6 @@ class FreehandMode (gui.mode.BrushworkModeMixin,
 
     def _reset_drawing_state(self):
         """Resets all per-TDW drawing state"""
-        self._remove_event_compression_workarounds()
         self._drawing_state = {}
 
     def _get_drawing_state(self, tdw):
@@ -255,7 +225,6 @@ class FreehandMode (gui.mode.BrushworkModeMixin,
     def enter(self, doc, **kwds):
         """Enter freehand mode"""
         super(FreehandMode, self).enter(doc, **kwds)
-        self._event_compression_supported = None
         self._drawing_state = {}
         self._reset_drawing_state()
         self._debug = (logger.getEffectiveLevel() == logging.DEBUG)
@@ -281,11 +250,14 @@ class FreehandMode (gui.mode.BrushworkModeMixin,
             return
         if not tdw.app.preferences.get("ui.hide_cursor_while_painting"):
             return
-        cursor = self._cursor_hidden
-        if not cursor:
-            cursor = gdk.Cursor(gdk.CursorType.BLANK_CURSOR)
+
+        if self._cursor_hidden is None:
+            window = tdw.get_window()
+            cursor = Gdk.Cursor.new_for_display(
+                window.get_display(), Gdk.CursorType.BLANK_CURSOR)
             self._cursor_hidden = cursor
-        tdw.set_override_cursor(cursor)
+
+        tdw.set_override_cursor(self._cursor_hidden)
         self._cursor_hidden_tdws.add(tdw)
 
     def _reinstate_drawing_cursor(self, tdw=None):
@@ -303,76 +275,13 @@ class FreehandMode (gui.mode.BrushworkModeMixin,
             tdw.set_override_cursor(None)
             self._cursor_hidden_tdws.remove(tdw)
 
-    ## Work around motion compression in recent GDKs
-
-    def _add_event_compression_workaround(self, tdw):
-        """Adds a workaround for the motion event compression bug"""
-        drawstate = self._get_drawing_state(tdw)
-        win = tdw.get_window()
-        msg_prefix = "Add motion event compression workaround: "
-        if (EVCOMPRESSION_WORKAROUND_ALLOW_DISABLE_VIA_API
-                and self._event_compression_supported):
-            workaround_used = EVCOMPRESSION_WORKAROUND_DISABLE_VIA_API
-            was_enabled = win.get_event_compression()
-            logger.debug(
-                msg_prefix
-                + "using set_event_compression(False) (%r) (was %r)",
-                tdw, was_enabled,
-            )
-            drawstate.event_compression_was_enabled = was_enabled
-            win.set_event_compression(False)
-        elif EVCOMPRESSION_WORKAROUND_ALLOW_EVHACK_FILTER:
-            workaround_used = EVCOMPRESSION_WORKAROUND_EVHACK_FILTER
-            assert drawstate.evhack_data is None
-            data = (tdw, self)
-            logger.warning(msg_prefix + "using evhack filter %r", data)
-            mypaintlib.evhack_gdk_window_add_filter(win, data)
-            drawstate.evhack_data = data
-            drawstate.evhack_positions = []
-        else:
-            workaround_used = EVCOMPRESSION_WORKAROUND_NONE
-            logger.warning(msg_prefix + "not using any workaround")
-        drawstate.event_compression_workaround = workaround_used
-
-    def _remove_event_compression_workarounds(self):
-        """Removes all workarounds for the motion event compression bug"""
-        msg_prefix = "Remove motion event compression workaround: "
-        for tdw, drawstate in self._drawing_state.iteritems():
-            win = tdw.get_window()
-            workaround_used = drawstate.event_compression_workaround
-            if workaround_used == EVCOMPRESSION_WORKAROUND_DISABLE_VIA_API:
-                mcomp = drawstate.event_compression_was_enabled
-                assert mcomp is not None
-                logger.debug(
-                    msg_prefix + "restoring event_compression to %r (%r)",
-                    mcomp, tdw,
-                )
-                win.set_event_compression(mcomp)
-            elif workaround_used == EVCOMPRESSION_WORKAROUND_EVHACK_FILTER:
-                drawstate = self._get_drawing_state(tdw)
-                data = drawstate.evhack_data
-                assert data is not None
-                logger.warning(msg_prefix + "removing evhack filter %r", data)
-                mypaintlib.evhack_gdk_window_remove_filter(win, data)
-                drawstate.evhack_data = None
-                drawstate.evhack_positions = []
-            else:
-                logger.warning(msg_prefix + "no workaround to disable")
-            drawstate.event_compression_workaround = None
-
-    def queue_evhack_position(self, tdw, x, y, t):
-        """Queues noncompressed motion data (called by eventhack.hpp)"""
-        if tdw.is_sensitive:
-            drawstate = self._get_drawing_state(tdw)
-            drawstate.evhack_positions.append((x, y, t))
-
     ## Input handlers
 
     def button_press_cb(self, tdw, event):
         result = False
         current_layer = tdw.doc.layer_stack.current
         if (current_layer.get_paintable() and event.button == 1
-                and event.type == gdk.BUTTON_PRESS):
+                and event.type == Gdk.EventType.BUTTON_PRESS):
             # Single button press
             # Stroke started, notify observers
             self.doc.input_stroke_started(event)
@@ -384,12 +293,16 @@ class FreehandMode (gui.mode.BrushworkModeMixin,
                 # "pressure" changes, so we simulate it. (Note: we can't
                 # use the event's button state because it carries the
                 # old state.)
-                self.motion_notify_cb(tdw, event, fakepressure=0.5)
+                self.motion_notify_cb(tdw, event,
+                                      fakepressure=tdw.app.fakepressure)
 
             drawstate.button_down = event.button
             drawstate.last_good_raw_pressure = 0.0
             drawstate.last_good_raw_xtilt = 0.0
             drawstate.last_good_raw_ytilt = 0.0
+            drawstate.last_good_raw_viewzoom = 0.0
+            drawstate.last_good_raw_viewrotation = 0.0
+            drawstate.last_good_raw_barrel_rotation = 0.0
 
             # Hide the cursor if configured to
             self._hide_drawing_cursor(tdw)
@@ -431,12 +344,6 @@ class FreehandMode (gui.mode.BrushworkModeMixin,
         Fake pressure is passed with faked motion events, e.g.
         button-press and button-release handlers for mouse events.
 
-        GTK 3.8 and above does motion compression, forcing our use of
-        event filter hackery to obtain the high-resolution event
-        positions required for making brushstrokes. This handler is
-        still called for the events the GDK compression code lets
-        through, and it is the only source of pressure and tilt info
-        available when motion compression is active.
         """
 
         # Do nothing if painting is inactivated
@@ -444,19 +351,10 @@ class FreehandMode (gui.mode.BrushworkModeMixin,
         if not (tdw.is_sensitive and current_layer.get_paintable()):
             return False
 
-        # Disable or work around GDK's motion event compression
-        if self._event_compression_supported is None:
-            win = tdw.get_window()
-            mc_supported = hasattr(win, "set_event_compression")
-            self._event_compression_supported = mc_supported
-        drawstate = self._get_drawing_state(tdw)
-        if drawstate.event_compression_workaround is None:
-            self._add_event_compression_workaround(tdw)
-
         # If the device has changed and the last pressure value from the
         # previous device is not equal to 0.0, this can leave a visible
         # stroke on the layer even if the 'new' device is not pressed on
-        # the tablet and has a pressure axis == 0.0.  Reseting the brush
+        # the tablet and has a pressure axis == 0.0.  Resetting the brush
         # when the device changes fixes this issue, but there may be a
         # much more elegant solution that only resets the brush on this
         # edge-case.
@@ -471,9 +369,12 @@ class FreehandMode (gui.mode.BrushworkModeMixin,
         x = event.x
         y = event.y
         time = event.time
-        pressure = event.get_axis(gdk.AXIS_PRESSURE)
-        xtilt = event.get_axis(gdk.AXIS_XTILT)
-        ytilt = event.get_axis(gdk.AXIS_YTILT)
+        pressure = event.get_axis(Gdk.AxisUse.PRESSURE)
+        xtilt = event.get_axis(Gdk.AxisUse.XTILT)
+        ytilt = event.get_axis(Gdk.AxisUse.YTILT)
+        viewzoom = tdw.scale
+        viewrotation = tdw.rotation
+        barrel_rotation = event.get_axis(Gdk.AxisUse.WHEEL)
         state = event.state
 
         # Workaround for buggy evdev behaviour.
@@ -482,16 +383,17 @@ class FreehandMode (gui.mode.BrushworkModeMixin,
         # lines. As a workaround, forbid zero pressures if there is a
         # button pressed down, and substitute the last-known good value.
         # Detail: https://github.com/mypaint/mypaint/issues/29
+        drawstate = self._get_drawing_state(tdw)
         if drawstate.button_down is not None:
             if pressure == 0.0:
                 pressure = drawstate.last_good_raw_pressure
-            elif pressure is not None and isfinite(pressure):
+            elif pressure is not None and np.isfinite(pressure):
                 drawstate.last_good_raw_pressure = pressure
 
-        # Ensure each non-evhack event has a defined pressure
+        # Ensure each event has a defined pressure
         if pressure is not None:
             # Using the reported pressure. Apply some sanity checks
-            if not isfinite(pressure):
+            if not np.isfinite(pressure):
                 # infinity/nan: use button state (instead of clamping in
                 # brush.hpp) https://gna.org/bugs/?14709
                 pressure = None
@@ -504,13 +406,15 @@ class FreehandMode (gui.mode.BrushworkModeMixin,
             if fakepressure is not None:
                 pressure = clamp(fakepressure, 0.0, 1.0)
             else:
-                pressure = (state & gdk.BUTTON1_MASK) and 0.5 or 0.0
+                pressure = (
+                    (state & Gdk.ModifierType.BUTTON1_MASK) and
+                    tdw.app.fakepressure or 0.0)
             drawstate.last_event_had_pressure = False
 
         # Check whether tilt is present.  For some tablets without
         # tilt support GTK reports a tilt axis with value nan, instead
         # of None.  https://gna.org/bugs/?17084
-        if xtilt is None or ytilt is None or not isfinite(xtilt+ytilt):
+        if xtilt is None or ytilt is None or not np.isfinite(xtilt + ytilt):
             xtilt = 0.0
             ytilt = 0.0
 
@@ -519,6 +423,28 @@ class FreehandMode (gui.mode.BrushworkModeMixin,
         xtilt = clamp(xtilt, -1.0, 1.0)
         ytilt = clamp(ytilt, -1.0, 1.0)
 
+        tilt_ascension = 0.5 * math.atan2(-xtilt, ytilt) / math.pi
+
+        # Offset barrel rotation if wanted
+        # This could be used to correct for different devices,
+        # Left vs Right handed, etc.
+        b_offset = tdw.app.preferences.get("input.barrel_rotation_offset")
+        if (barrel_rotation is not None):
+            barrel_rotation = (barrel_rotation + b_offset) % 1.0
+
+        # barrel_rotation is likely affected by ascension (a bug?)
+        # lets compensate but allow disabling
+        if (barrel_rotation is not None and tdw.app.preferences.get(
+           "input.barrel_rotation_subtract_ascension")):
+            barrel_rotation = (barrel_rotation - tilt_ascension) % 1.0
+
+        # If WHEEL is missing (barrel_rotation)
+        # Use the fakerotation controller to allow keyboard control
+        # We can't trust None so also look at preference
+        if (barrel_rotation is None
+           or not tdw.app.preferences.get("input.use_barrel_rotation")):
+            barrel_rotation = (tdw.app.fakerotation + b_offset) % 1.0
+
         # Evdev workaround. X and Y tilts suffer from the same
         # problem as pressure for fancier devices.
         if drawstate.button_down is not None:
@@ -531,24 +457,9 @@ class FreehandMode (gui.mode.BrushworkModeMixin,
             else:
                 drawstate.last_good_raw_ytilt = ytilt
 
-        # Tilt inputs are assumed to be relative to the viewport,
-        # but the canvas may be rotated or mirrored, or both.
-        # Compensate before passing them to the brush engine.
-        # https://gna.org/bugs/?19988
         if tdw.mirrored:
             xtilt *= -1.0
-        if tdw.rotation != 0:
-            tilt_angle = math.atan2(ytilt, xtilt) - tdw.rotation
-            tilt_magnitude = math.sqrt((xtilt**2) + (ytilt**2))
-            xtilt = tilt_magnitude * math.cos(tilt_angle)
-            ytilt = tilt_magnitude * math.sin(tilt_angle)
-
-        # HACK: color picking, do not paint
-        # TEST: Does this ever happen now?
-        if state & gdk.CONTROL_MASK or state & gdk.MOD1_MASK:
-            # Don't simply return; this is a workaround for unwanted
-            # lines in https://gna.org/bugs/?16169
-            pressure = 0.0
+            barrel_rotation *= -1.0
 
         # Apply pressure mapping if we're running as part of a full
         # MyPaint application (and if there's one defined).
@@ -561,42 +472,20 @@ class FreehandMode (gui.mode.BrushworkModeMixin,
         else:
             self._reinstate_drawing_cursor(tdw)
 
-        # HACK: straight line mode?
-        # TEST: Does this ever happen?
-        if state & gdk.SHIFT_MASK:
-            pressure = 0.0
-
-        # If the eventhack filter caught more than one event, push them
-        # onto the motion event queue. Pressures and tilts will be
-        # interpolated from surrounding motion-notify events.
-        if len(drawstate.evhack_positions) > 1:
-            # Remove the last item: it should be the one corresponding
-            # to the current motion-notify-event.
-            hx0, hy0, ht0 = drawstate.evhack_positions.pop(-1)
-            # Check that we can use the eventhack data uncorrected
-            if (hx0, hy0, ht0) == (x, y, time):
-                for hx, hy, ht in drawstate.evhack_positions:
-                    hx, hy = tdw.display_to_model(hx, hy)
-                    event_data = (ht, hx, hy, None, None, None)
-                    drawstate.queue_motion(event_data)
-            else:
-                logger.warning(
-                    "Final evhack event (%0.2f, %0.2f, %d) doesn't match its "
-                    "corresponding motion-notify-event (%0.2f, %0.2f, %d). "
-                    "This can be ignored if it's just a one-off occurrence.",
-                    hx0, hy0, ht0, x, y, time)
-        # Reset the eventhack queue
-        if len(drawstate.evhack_positions) > 0:
-            drawstate.evhack_positions = []
-
         # Queue this event
         x, y = tdw.display_to_model(x, y)
-        event_data = (time, x, y, pressure, xtilt, ytilt)
+
+        event_data = (time, x, y, pressure,
+                      xtilt, ytilt, viewzoom,
+                      viewrotation, barrel_rotation)
         drawstate.queue_motion(event_data)
         # Start the motion event processor, if it isn't already running
         if not drawstate.motion_processing_cbid:
-            cbid = gobject.idle_add(self._motion_queue_idle_cb, tdw,
-                                    priority=self.MOTION_QUEUE_PRIORITY)
+            cbid = GLib.idle_add(
+                self._motion_queue_idle_cb,
+                tdw,
+                priority = self.MOTION_QUEUE_PRIORITY,
+            )
             drawstate.motion_processing_cbid = cbid
 
     ## Motion queue processing
@@ -621,7 +510,8 @@ class FreehandMode (gui.mode.BrushworkModeMixin,
     def _process_queued_event(self, tdw, event_data):
         """Process one motion event from the motion queue"""
         drawstate = self._get_drawing_state(tdw)
-        time, x, y, pressure, xtilt, ytilt = event_data
+        (time, x, y, pressure, xtilt, ytilt, viewzoom,
+         viewrotation, barrel_rotation) = event_data
         model = tdw.doc
 
         # Calculate time delta for the brush engine
@@ -629,26 +519,25 @@ class FreehandMode (gui.mode.BrushworkModeMixin,
         drawstate.last_handled_event_time = time
         if not last_event_time:
             return
-        dtime = (time - last_event_time)/1000.0
+        dtime = (time - last_event_time) / 1000.0
         if self._debug:
             cavg = drawstate.avgtime
             if cavg is not None:
                 tavg, nevents = cavg
                 nevents += 1
-                tavg += (dtime - tavg)/nevents
+                tavg += (dtime - tavg) / nevents
             else:
                 tavg = dtime
                 nevents = 1
-            if nevents*tavg > 1.0 and nevents > 20:
+            if ((nevents * tavg) > 1.0) and nevents > 20:
                 logger.debug("Processing at %d events/s (t_avg=%0.3fs)",
                              nevents, tavg)
                 drawstate.avgtime = None
             else:
                 drawstate.avgtime = (tavg, nevents)
 
-        # Refuse drawing if the layer is locked or hidden
         current_layer = model._layers.current
-        if current_layer.locked or not current_layer.visible:
+        if not current_layer.get_paintable():
             return
 
         # Feed data to the brush engine.  Pressure and tilt cleanup
@@ -659,9 +548,11 @@ class FreehandMode (gui.mode.BrushworkModeMixin,
         # https://github.com/mypaint/mypaint/issues/344
 
         pressure = clamp(pressure, 0.0, 1.0)
-        tilt = clamp(xtilt, -1.0, 1.0)
+        xtilt = clamp(xtilt, -1.0, 1.0)
         ytilt = clamp(ytilt, -1.0, 1.0)
-        self.stroke_to(model, dtime, x, y, pressure, xtilt, ytilt)
+        self.stroke_to(model, dtime, x, y, pressure,
+                       xtilt, ytilt, viewzoom,
+                       viewrotation, barrel_rotation)
 
         # Update the TDW's idea of where we last painted
         # FIXME: this should live in the model, not the view
@@ -684,15 +575,51 @@ class FreehandOptionsWidget (gui.mode.PaintingModeOptionsWidgetBase):
 
     def init_specialized_widgets(self, row):
         cname = "slow_tracking"
-        label = gtk.Label()
-        #TRANSLATORS: Short alias for "Slow position tracking". This is
-        #TRANSLATORS: used on the options panel.
+        label = Gtk.Label()
+        # TRANSLATORS: Short alias for "Slow position tracking". This is
+        # TRANSLATORS: used on the options panel.
         label.set_text(_("Smooth:"))
         label.set_alignment(1.0, 0.5)
         label.set_hexpand(False)
         self.adjustable_settings.add(cname)
         adj = self.app.brush_adjustment[cname]
-        scale = gtk.HScale(adj)
+        scale = Gtk.Scale.new(Gtk.Orientation.HORIZONTAL, adj)
+        scale.set_draw_value(False)
+        scale.set_hexpand(True)
+        self.attach(label, 0, row, 1, 1)
+        self.attach(scale, 1, row, 1, 1)
+        row += 1
+        cname = "fakepressure"
+        label = Gtk.Label()
+        # TRANSLATORS: Short alias for "Fake Pressure (for mouse)". This is
+        # TRANSLATORS: used on the options panel.
+        label.set_text(_("Pressure:"))
+        label.set_alignment(1.0, 0.5)
+        label.set_hexpand(False)
+        changed_cb = self._fakepressure_value_changed_cb
+        adj = Gtk.Adjustment(value=0.5, lower=0.0, upper=1.0,
+                             step_increment=0.01, page_increment=0.1)
+        self.app.fake_adjustment['fakepressure'] = adj
+        adj.connect("value-changed", changed_cb)
+        scale = Gtk.Scale.new(Gtk.Orientation.HORIZONTAL, adj)
+        scale.set_draw_value(False)
+        scale.set_hexpand(True)
+        self.attach(label, 0, row, 1, 1)
+        self.attach(scale, 1, row, 1, 1)
+        row += 1
+        cname = "fakerotation"
+        label = Gtk.Label()
+        # TRANSLATORS: Short alias for "Fake Barrel Rotation (for mouse)".
+        # TRANSLATORS: This is used on the options panel.
+        label.set_text(_("Twist:"))
+        label.set_alignment(1.0, 0.5)
+        label.set_hexpand(False)
+        changed_cb = self._fakerotation_value_changed_cb
+        adj = Gtk.Adjustment(value=0.5, lower=0.0, upper=1.0,
+                             step_increment=0.0625, page_increment=0.25)
+        self.app.fake_adjustment['fakerotation'] = adj
+        adj.connect("value-changed", changed_cb)
+        scale = Gtk.Scale.new(Gtk.Orientation.HORIZONTAL, adj)
         scale.set_draw_value(False)
         scale.set_hexpand(True)
         self.attach(label, 0, row, 1, 1)
@@ -700,6 +627,28 @@ class FreehandOptionsWidget (gui.mode.PaintingModeOptionsWidgetBase):
         row += 1
         return row
 
+    def _fakepressure_value_changed_cb(self, adj):
+        """Updates fakepressure when the user tweaks it using a scale"""
+        newvalue = adj.get_value()
+        self.app.fakepressure = newvalue
+
+    def fakepressure_modified_cb(self, value):
+        """Updates the fakepressure slider when changed elsewhere"""
+        adj = self.app.fake_adjustment.get('fakepressure', None)
+        if adj is not None:
+            adj.set_value(value)
+
+    def _fakerotation_value_changed_cb(self, adj):
+        """Updates fakerotation when the user tweaks it using a scale"""
+        newvalue = adj.get_value()
+        self.app.fakerotation = newvalue
+
+    def fakerotation_modified_cb(self, value):
+        """Updates the fakerotation slider when changed elsewhere"""
+        adj = self.app.fake_adjustment.get('fakerotation', None)
+        if adj is not None:
+            adj.set_value(value)
+
 
 class PressureAndTiltInterpolator (object):
     """Interpolates event sequences, filling in null pressure/tilt data
@@ -709,9 +658,81 @@ class PressureAndTiltInterpolator (object):
     nice tailoff for mouse users. The interpolator is sensitive to
     transitions between nonzero and zero effective pressure in both
     directions. These transitions clear out just enough history to avoid
-    hook-off and lead-in artefacts.
+    hook-off and lead-in artifacts.
+
+    >>> interp = PressureAndTiltInterpolator()
+    >>> raw_data = interp._TEST_DATA
+    >>> all([len(t) == 9 for t in raw_data])
+    True
+    >>> any([t for t in raw_data if None in t[3:]])
+    True
+    >>> cooked_data = []
+    >>> for raw_event in raw_data:
+    ...    for cooked_event in interp.feed(*raw_event):
+    ...        cooked_data.append(cooked_event)
+    >>> any([t for t in cooked_data if None in t[3:]])
+    False
+    >>> len(cooked_data) <= len(raw_data)
+    True
+    >>> all([(t in cooked_data) for t in raw_data
+    ...      if None not in t[3:]])
+    True
+    >>> any([t for t in cooked_data if 70 < t[0] < 110])
+    False
+    >>> len([t for t in cooked_data if t[0] in (70, 110)]) == 2
+    True
+
     """
 
+    # Test data:
+
+    _TEST_DATA = [
+        # These 2 events will be dropped (no prior state with pressure).
+        (3, 0.3, 0.3, None, None, None, None, None, None),
+        (7, 0.7, 0.7, None, None, None, None, None, None),
+        (10, 1.0, 1.0, 0.33, 0.0, 0.5, 1.0, 0.0, 0.0),
+        # Gaps between defined data like this one will have those
+        # None entries filled in.
+        (13, 1.3, 1.3, None, None, None, None, None, None),
+        (15, 1.5, 1.5, None, None, None, None, None, None),
+        (17, 1.7, 1.7, None, None, None, None, None, None),
+        (20, 2.0, 2.0, 0.45, 0.1, 0.4, 1.0, 0.0, 0.0),
+        (23, 2.3, 2.3, None, None, None, None, None, None),
+        (27, 2.7, 2.7, None, None, None, None, None, None),
+        (30, 3.0, 3.0, 0.50, 0.2, 0.3, 1.0, 0.0, 0.0),
+        (33, 3.3, 3.3, None, None, None, None, None, None),
+        (37, 3.7, 3.7, None, None, None, None, None, None),
+        (40, 4.0, 4.0, 0.40, 0.3, 0.2, 1.0, 0.0, 0.0),
+        (44, 4.4, 4.4, None, None, None, None, None, None),
+        (47, 4.7, 4.7, None, None, None, None, None, None),
+        (50, 5.0, 5.0, 0.30, 0.5, 0.1, 1.0, 0.0, 0.0),
+        (53, 5.3, 5.3, None, None, None, None, None, None),
+        (57, 5.7, 5.7, None, None, None, None, None, None),
+        (60, 6.0, 6.0, 0.11, 0.4, 0.0, 1.0, 0.0, 0.0),
+        (63, 6.3, 6.3, None, None, None, None, None, None),
+        (67, 6.7, 6.7, None, None, None, None, None, None),
+        # Down to zero pressure...
+        (70, 7.0, 7.0, 0.00, 0.2, 0.0, 1.0, 0.0, 0.0),
+        # .. followed by a null-pressure sequence.
+        # That means that this gap will be skipped over till an
+        # event with a defined pressure comes along.
+        (73, 7.0, 7.0, None, None, None, None, None, None),
+        (78, 50.0, 50.0, None, None, None, None, None, None),
+        (83, 110.0, 110.0, None, None, None, None, None, None),
+        (88, 120.0, 120.0, None, None, None, None, None, None),
+        (93, 130.0, 130.0, None, None, None, None, None, None),
+        (98, 140.0, 140.0, None, None, None, None, None, None),
+        (103, 150.0, 150.0, None, None, None, None, None, None),
+        (108, 160.0, 160.0, None, None, None, None, None, None),
+        # Normally, event tuples won't be altered or have extra events
+        # inserted between them.
+        (110, 170.0, 170.0, 0.11, 0.1, 0.0, 1.0, 0.0, 0.0),
+        (120, 171.0, 171.0, 0.33, 0.0, 0.0, 1.0, 0.0, 0.0),
+        (130, 172.0, 172.0, 0.00, 0.0, 0.0, 1.0, 0.0, 0.0)
+    ]
+
+    # Construction:
+
     def __init__(self):
         """Instantiate with a clear internal state"""
         object.__init__(self)
@@ -724,6 +745,8 @@ class PressureAndTiltInterpolator (object):
         self._np = []
         self._np_next = []
 
+    # Internals:
+
     def _clear(self):
         """Reset to the initial clean state"""
         self._pt0_prev = None
@@ -758,14 +781,14 @@ class PressureAndTiltInterpolator (object):
             dt = t1 - t0
             can_interp = dt > 0
         if can_interp:
-            for np in self._np:
-                t, x, y = np[0:3]
-                p, xt, yt = spline_4p(
-                    float(t - t0) / dt,
-                    array(pt0p[3:]), array(pt0[3:]),
-                    array(pt1[3:]), array(pt1n[3:])
+            for event in self._np:
+                t, x, y = event[0:3]
+                p, xt, yt, vz, vr, br = spline_4p(
+                    (t - t0) / dt,
+                    np.array(pt0p[3:]), np.array(pt0[3:]),
+                    np.array(pt1[3:]), np.array(pt1n[3:])
                 )
-                yield (t, x, y, p, xt, yt)
+                yield (t, x, y, p, xt, yt, vz, vr, br)
         if pt1 is not None:
             yield pt1
 
@@ -777,7 +800,7 @@ class PressureAndTiltInterpolator (object):
                 (self._pt1 is not None) and
                 (self._pt1[3] <= 0.0)):
             # Transitions from zero to nonzero pressure
-            # Clear history to avoid artefacts
+            # Clear history to avoid artifacts
             self._pt0_prev = None   # ignore the current pt0
             self._pt0 = self._pt1
             self._pt1 = self._pt1_next
@@ -798,7 +821,10 @@ class PressureAndTiltInterpolator (object):
             # Normal forward of control points and event buffers
             self._step()
 
-    def feed(self, time, x, y, pressure, xtilt, ytilt):
+    # Public methods:
+
+    def feed(self, time, x, y, pressure, xtilt, ytilt, viewzoom,
+             viewrotation, barrel_rotation):
         """Feed in an event, yielding zero or more interpolated events
 
         :param time: event timestamp, integer number of milliseconds
@@ -809,59 +835,37 @@ class PressureAndTiltInterpolator (object):
         :param pressure: Effective pen pressure, [0.0, 1.0]
         :param xtilt: Pen tilt in the model X direction, [-1.0, 1.0]
         :param ytilt: Pen tilt in the model's Y direction, [-1.0, 1.0]
+        :param viewzoom: The view's current zoom level, [0, 64]
+        :param viewrotation: The view's current rotation, [-180.0, 180.0]
+        :param barrel_rotation: The stylus barrel rotation, [0.0, 1.0]
         :returns: Iterator of event tuples
 
-        Event tuples have the form (TIME, X, Y, PRESSURE, XTILT, YTILT).
+        Event tuples have the form (TIME, X, Y, PRESSURE, XTILT, YTILT,
+        VIEWZOOM, VIEWROTATION, BARREL_ROTATION).
         """
-        if None in (pressure, xtilt, ytilt):
-            self._np_next.append((time, x, y, pressure, xtilt, ytilt))
+        if None in (pressure, xtilt, ytilt, viewzoom,
+                    viewrotation, barrel_rotation):
+            self._np_next.append((time, x, y, pressure, xtilt, ytilt, viewzoom,
+                                  viewrotation, barrel_rotation))
         else:
-            self._pt1_next = (time, x, y, pressure, xtilt, ytilt)
-            for t, x, y, p, xt, yt in self._interpolate_and_step():
-                yield (t, x, y, p, xt, yt)
+            self._pt1_next = (time, x, y, pressure, xtilt, ytilt, viewzoom,
+                              viewrotation, barrel_rotation)
+            for t, x, y, p, xt, yt, vz, vr, br in self._interpolate_and_step():
+                yield (t, x, y, p, xt, yt, vz, vr, br)
 
 
 ## Module tests
 
-if __name__ == '__main__':
+def _test():
+    import doctest
+    doctest.testmod()
     interp = PressureAndTiltInterpolator()
-    events = [
-        (3, 0.3, 0.3, None, None, None),
-        (7, 0.7, 0.7, None, None, None),
-        (10, 1.0, 1.0, 0.33, 0.0, 0.5),
-        (13, 1.3, 1.3, None, None, None),
-        (15, 1.5, 1.5, None, None, None),
-        (17, 1.7, 1.7, None, None, None),
-        (20, 2.0, 2.0, 0.45, 0.1, 0.4),
-        (23, 2.3, 2.3, None, None, None),
-        (27, 2.7, 2.7, None, None, None),
-        (30, 3.0, 3.0, 0.50, 0.2, 0.3),
-        (33, 3.3, 3.3, None, None, None),
-        (37, 3.7, 3.7, None, None, None),
-        (40, 4.0, 4.0, 0.40, 0.3, 0.2),
-        (44, 4.4, 4.4, None, None, None),
-        (47, 4.7, 4.7, None, None, None),
-        (50, 5.0, 5.0, 0.30, 0.5, 0.1),
-        (53, 5.3, 5.3, None, None, None),
-        (57, 5.7, 5.7, None, None, None),
-        (60, 6.0, 6.0, 0.11, 0.4, 0.0),
-        (63, 6.3, 6.3, None, None, None),
-        (67, 6.7, 6.7, None, None, None),
-        (70, 7.0, 7.0, 0.00, 0.2, 0.0),
-        (73, 7.0, 7.0, None, None, None),
-        (78, 50.0, 50.0, None, None, None),
-        (83, 110.0, 110.0, None, None, None),
-        (88, 120.0, 120.0, None, None, None),
-        (93, 130.0, 130.0, None, None, None),
-        (98, 140.0, 140.0, None, None, None),
-        (103, 150.0, 150.0, None, None, None),
-        (108, 160.0, 160.0, None, None, None),
-        (110, 170.0, 170.0, 0.11, 0.1, 0.0),
-        (120, 171.0, 171.0, 0.33, 0.0, 0.0),
-        (130, 172.0, 172.0, 0.00, 0.0, 0.0)
-    ]
     # Emit CSV for ad-hoc plotting
-    print "time,x,y,pressure,xtilt,ytilt"
-    for event in events:
+    print("time,x,y,pressure,xtilt,ytilt,viewzoom,viewrotation,barrel_rotation")
+    for event in interp._TEST_DATA:
         for data in interp.feed(*event):
-            print ",".join([str(c) for c in data])
+            print(",".join([str(c) for c in data]))
+
+
+if __name__ == '__main__':
+    _test()
