@@ -1,6 +1,6 @@
 # This file is part of MyPaint.
 # Copyright (C) 2008-2013 by Martin Renold <martinxyz@gmx.ch>
-# Copyright (C) 2008-2015 by the MyPaint Development Team
+# Copyright (C) 2008-2018 by the MyPaint Development Team
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -10,38 +10,35 @@
 
 ## Imports
 
-import gtk2compat
-import gobject
-import cairo
-import gtk
-from gtk import gdk
-import glib
-
-import os
+from __future__ import division, print_function
 import random
 from math import floor, ceil, log, exp
 import math
-from numpy import isfinite
-from numpy import empty
-from warnings import warn
 import weakref
 import contextlib
 import logging
-logger = logging.getLogger(__name__)
+
+from gi.repository import Gtk
+from gi.repository import Gdk
+from gi.repository import GLib
+import cairo
+import numpy as np
 
 from lib import helpers, tiledsurface, pixbufsurface
 from lib.observable import event
 import lib.layer
-import cursor
-from drawutils import render_checks
+from . import cursor
+from .drawutils import render_checks
 import gui.style
 import lib.color
+from lib.pycompat import xrange
 
+logger = logging.getLogger(__name__)
 
 ## Class definitions
 
 
-class TiledDrawWidget (gtk.EventBox):
+class TiledDrawWidget (Gtk.EventBox):
     """Widget for showing a lib.document.Document
 
     Rendering is delegated to a dedicated class: see `CanvasRenderer`.
@@ -55,7 +52,7 @@ class TiledDrawWidget (gtk.EventBox):
     __tdw_refs = []
 
     @classmethod
-    def get_active_tdw(kin):
+    def get_active_tdw(kin):  # noqa: N804
         """Returns the most recently created or entered TDW.
         """
         # Find and return the first visible, mapped etc. TDW in the list
@@ -128,24 +125,24 @@ class TiledDrawWidget (gtk.EventBox):
         """Instantiate a TiledDrawWidget.
 
         """
-        gtk.EventBox.__init__(self)
+        super(TiledDrawWidget, self).__init__()
 
         if __name__ == '__main__':
             app = None
         else:
-            import application
+            from . import application
             app = application.get_app()
         self.app = app
         self.doc = None
 
         self.add_events(
-            gdk.BUTTON_PRESS_MASK |
-            gdk.BUTTON_RELEASE_MASK |
-            gdk.POINTER_MOTION_MASK
+            Gdk.EventMask.BUTTON_PRESS_MASK |
+            Gdk.EventMask.BUTTON_RELEASE_MASK |
+            Gdk.EventMask.POINTER_MOTION_MASK
         )
         # Support smooth scrolling unless configured not to
         if app and app.preferences.get("ui.support_smooth_scrolling", True):
-            self.add_events(gdk.SMOOTH_SCROLL_MASK)
+            self.add_events(Gdk.EventMask.SMOOTH_SCROLL_MASK)
 
         self.last_painting_pos = None
 
@@ -156,19 +153,23 @@ class TiledDrawWidget (gtk.EventBox):
         self.add(self.renderer)
         self.renderer.update_cursor()  # get the initial cursor right
 
-        self.add_events(gdk.ENTER_NOTIFY_MASK)
+        self.add_events(Gdk.EventMask.ENTER_NOTIFY_MASK)
         self.connect("enter-notify-event", self.enter_notify_cb)
         self.__tdw_refs.insert(0, weakref.ref(self))
 
         self._last_alloc_pos = (0, 0)
         self.connect("size-allocate", self._size_allocate_cb)
+        self.connect("realize", self._realize_cb)
 
         #: Scroll to match appearing/disappearing sidebars and toolbars.
         self.scroll_on_allocate = True
 
-        forwarder = lambda *a: self.transformation_updated()
+        forwarder = self._announce_transformation_updated
         self.renderer.transformation_updated += forwarder
 
+    def _announce_transformation_updated(self, *args):
+        self.transformation_updated()
+
     @event
     def transformation_updated(self):
         """Forwarded event: transformation was updated"""
@@ -199,6 +200,11 @@ class TiledDrawWidget (gtk.EventBox):
             dy = new_pos[1] - old_pos[1]
             self.renderer.scroll(dx, dy, ongoing=False)
 
+    def _realize_cb(self, widget):
+        logger.debug("Turning off event compression for %r's window", widget)
+        win = widget.get_window()
+        win.set_event_compression(False)
+
     def set_model(self, model):
         assert self.doc is None
         renderer = self.renderer
@@ -346,7 +352,7 @@ class TiledDrawWidget (gtk.EventBox):
         y = alloc.y
         w = alloc.width
         h = alloc.height
-        corners = [(x, y), (x+w, y), (x+w, y+h), (x, y+h)]
+        corners = [(x, y), (x + w, y), (x + w, y + h), (x, y + h)]
         corners = [self.display_to_model(*p) for p in corners]
         return corners
 
@@ -472,7 +478,7 @@ class TiledDrawWidget (gtk.EventBox):
         :param tuple edge_p1: point on the edge, as model (x, y)
         :param tuple edge_p2: point on the edge, as model (x, y)
         :param int tolerance: slack for cursor pos., in display pixels
-        :patam bool finite: if false, the edge extends beyond p1, p2
+        :param bool finite: if false, the edge extends beyond p1, p2
         :returns: move direction cursor & distance from line (cursor, d)
         :rtype: tuple
 
@@ -492,31 +498,32 @@ class TiledDrawWidget (gtk.EventBox):
         x0, y0 = cursor_pos
         x1, y1 = self.model_to_display(*edge_p1)
         x2, y2 = self.model_to_display(*edge_p2)
-        Dx = x2 - x1
-        Dy = y2 - y1
-        edge_len = math.sqrt(Dx**2 + Dy**2)
+        dx = x2 - x1
+        dy = y2 - y1
+        edge_len = math.sqrt(dx**2 + dy**2)
         if edge_len <= 0:
-            dist1 = math.hypot(x0-x1, y0-y1)
-            dist2 = math.hypot(x0-x2, y0-y2)
+            dist1 = math.hypot(x0 - x1, y0 - y1)
+            dist2 = math.hypot(x0 - x2, y0 - y2)
             return (None, min(dist1, dist2))
         # Perpendicular distance from a line.
-        two_triarea = abs(Dy*x0 - Dx*y0 - x1*y2 + x2*y1)
+        two_triarea = abs((dy * x0) - (dx * y0) - (x1 * y2) + (x2 * y1))
         perp_dist = two_triarea / edge_len
         if perp_dist > tolerance:
             return (None, perp_dist)
         # Rough approximation here, but good enough for hit calculation.
         # Omit points too far away from the edge's centre point.
         if finite:
-            xmid = (x1 + x2)/2.0
-            ymid = (y1 + y2)/2.0
-            dist_from_midpt = math.sqrt((xmid-x0)**2 + (ymid-y0)**2)
-            if dist_from_midpt > (edge_len/2.0)+tolerance:
+            xmid = (x1 + x2) / 2.0
+            ymid = (y1 + y2) / 2.0
+            dist_from_midpt = math.sqrt(((xmid - x0) ** 2) +
+                                        ((ymid - y0) ** 2))
+            if dist_from_midpt > ((edge_len / 2.0) + tolerance):
                 return (None, perp_dist)
         # Cursor name by sector.
         # Aiming for a cursor that looks perpendicular to the line.
         # Ish.
-        theta = math.atan2(Dy, Dx)
-        c = cursor.get_move_cursor_name_for_angle(theta + math.pi/2)
+        theta = math.atan2(dy, dx)
+        c = cursor.get_move_cursor_name_for_angle(theta + (math.pi / 2))
         return (c, perp_dist)
 
 
@@ -542,7 +549,7 @@ class CanvasTransformation (object):
 class DrawCursorMixin(object):
     """Mixin for renderer widgets needing a managed drawing cursor.
 
-    Required members: self.doc, self.scale, gtk.Widget stuff.
+    Required members: self.doc, self.scale, Gtk.Widget stuff.
 
     """
 
@@ -577,10 +584,11 @@ class DrawCursorMixin(object):
         elif self.doc is None:
             logger.error("update_cursor: no document")
             return
-        elif layer.locked or not layer.visible or not layer.get_paintable():
+        elif not layer.get_paintable():
             # Cursor to represent that one cannot draw.
             # Often a red circle with a diagonal bar through it.
-            c = gdk.Cursor(gdk.CIRCLE)
+            c = Gdk.Cursor.new_for_display(
+                window.get_display(), Gdk.CursorType.CIRCLE)
         elif app is None:
             logger.error("update_cursor: no app")
             return
@@ -602,7 +610,7 @@ class DrawCursorMixin(object):
         choose normally again.
         """
         self._override_cursor = cursor
-        gobject.idle_add(self.update_cursor)
+        GLib.idle_add(self.update_cursor)
 
     def _get_cursor_info(self):
         """Return factors determining the cursor size and shape.
@@ -632,14 +640,15 @@ class DrawCursorMixin(object):
             self.update_cursor()
 
 
-def calculate_transformation_matrix(scale, rotation, translation_x, translation_y, mirrored):
-
+def calculate_transformation_matrix(scale, rotation,
+                                    translation_x, translation_y,
+                                    mirrored):
     scale = scale
     # check if scale is almost a power of two
     scale_log2 = log(scale, 2)
     scale_log2_rounded = round(scale_log2)
-    if abs(scale_log2-scale_log2_rounded) < 0.01:
-        scale = 2.0**scale_log2_rounded
+    if abs(scale_log2 - scale_log2_rounded) < 0.01:
+        scale = 2.0 ** scale_log2_rounded
 
     # maybe we should check if rotation is almost a multiple of 90 degrees?
 
@@ -665,7 +674,7 @@ def calculate_transformation_matrix(scale, rotation, translation_x, translation_
     return matrix
 
 
-class CanvasRenderer (gtk.DrawingArea, DrawCursorMixin):
+class CanvasRenderer (Gtk.DrawingArea, DrawCursorMixin):
     """Render the document model to screen.
 
     Can render the document in a transformed way, including translation,
@@ -676,7 +685,7 @@ class CanvasRenderer (gtk.DrawingArea, DrawCursorMixin):
     ## Method defs
 
     def __init__(self, tdw, idle_redraw_priority=None):
-        gtk.DrawingArea.__init__(self)
+        super(CanvasRenderer, self).__init__()
         self.init_draw_cursor()
 
         self.connect("draw", self._draw_cb)
@@ -711,7 +720,7 @@ class CanvasRenderer (gtk.DrawingArea, DrawCursorMixin):
 
         # gets overwritten for the main window
         self.zoom_max = 5.0
-        self.zoom_min = 1/5.0
+        self.zoom_min = 1 / 5.0
 
         # Sensitivity; we draw via a cached snapshot while the widget is
         # insensitive. tdws are generally only insensitive during loading and
@@ -737,14 +746,16 @@ class CanvasRenderer (gtk.DrawingArea, DrawCursorMixin):
         self._hq_rendering = True
         self._restore_hq_rendering_timeout_id = None
 
+        self.connect("configure-event", self._configure_event_cb)
+
     def _init_alpha_checks(self):
         """Initialize the alpha check backgrounds"""
         # Real: checkerboard pattern, rendered via Cairo
         assert tiledsurface.N % gui.style.ALPHA_CHECK_SIZE == 0
-        N = tiledsurface.N
+        n = tiledsurface.N
         size = gui.style.ALPHA_CHECK_SIZE
-        nchecks = int(N / size)
-        cairo_surf = cairo.ImageSurface(cairo.FORMAT_ARGB32, N, N)
+        nchecks = int(n // size)
+        cairo_surf = cairo.ImageSurface(cairo.FORMAT_ARGB32, n, n)
         cr = cairo.Context(cairo_surf)
         render_checks(cr, size, nchecks)
         cairo_surf.flush()
@@ -753,16 +764,18 @@ class CanvasRenderer (gtk.DrawingArea, DrawCursorMixin):
         pattern.set_extend(cairo.EXTEND_REPEAT)
         self._real_alpha_check_pattern = pattern
         # Fake: faster rendering, but ugly
-        tile = empty((N, N, 4), dtype='uint16')
+        tile = np.empty((n, n, 4), dtype='uint16')
         f = 1 << 15
         col1 = [int(f * c) for c in gui.style.ALPHA_CHECK_COLOR_1] + [f]
         col2 = [int(f * c) for c in gui.style.ALPHA_CHECK_COLOR_2] + [f]
         tile[:] = col1
         for i in xrange(nchecks):
             for j in xrange(nchecks):
-                if (i+j) % 2 == 0:
+                if (i + j) % 2 == 0:
                     continue
-                tile[i*size:(i+1)*size, j*size:(j+1)*size] = col2
+                ia, ib = (i * size), ((i + 1) * size)
+                ja, jb = (j * size), ((j + 1) * size)
+                tile[ia:ib, ja:jb] = col2
         self._fake_alpha_check_tile = tile
 
     @property
@@ -830,7 +843,7 @@ class CanvasRenderer (gtk.DrawingArea, DrawCursorMixin):
         something).
 
         """
-        insensitive = self.get_state_flags() & gtk.StateFlags.INSENSITIVE
+        insensitive = self.get_state_flags() & Gtk.StateFlags.INSENSITIVE
         if insensitive and (not self._insensitive_state_content):
             alloc = widget.get_allocation()
             w = alloc.width
@@ -854,25 +867,25 @@ class CanvasRenderer (gtk.DrawingArea, DrawCursorMixin):
 
         if w == 0 and h == 0:
             # Full redraw (used when background has changed).
-            #logger.debug('Full redraw')
+            # logger.debug('Full redraw')
             self.queue_draw()
             return
 
         # Create an expose event with the event bbox rotated/zoomed.
-        corners = [(x, y), (x+w, y), (x, y+h), (x+w, y+h)]
+        corners = [(x, y), (x + w, y), (x, y + h), (x + w, y + h)]
         corners = [self.model_to_display(x, y) for (x, y) in corners]
         bbox = helpers.rotated_rectangle_bbox(corners)
         self.queue_draw_area(*bbox)
 
     def queue_draw(self):
         if self._idle_redraw_priority is None:
-            gtk.DrawingArea.queue_draw(self)
+            super(CanvasRenderer, self).queue_draw()
             return
         self._queue_idle_redraw(None)
 
     def queue_draw_area(self, x, y, w, h):
         if self._idle_redraw_priority is None:
-            gtk.DrawingArea.queue_draw_area(self, x, y, w, h)
+            super(CanvasRenderer, self).queue_draw_area(x, y, w, h)
             return
         bbox = helpers.Rect(x, y, w, h)
         self._queue_idle_redraw(bbox)
@@ -891,7 +904,7 @@ class CanvasRenderer (gtk.DrawingArea, DrawCursorMixin):
         queue.append(bbox)
         if self._idle_redraw_src_id is not None:
             return
-        src_id = glib.idle_add(
+        src_id = GLib.idle_add(
             self._idle_redraw_cb,
             priority = self._idle_redraw_priority,
         )
@@ -903,9 +916,9 @@ class CanvasRenderer (gtk.DrawingArea, DrawCursorMixin):
         if len(queue) > 0:
             bbox = queue.pop(0)
             if bbox is None:
-                gtk.DrawingArea.queue_draw(self)
+                super(CanvasRenderer, self).queue_draw()
             else:
-                gtk.DrawingArea.queue_draw_area(self, *bbox)
+                super(CanvasRenderer, self).queue_draw_area(*bbox)
         if len(queue) == 0:
             self._idle_redraw_src_id = None
             return False
@@ -943,8 +956,10 @@ class CanvasRenderer (gtk.DrawingArea, DrawCursorMixin):
 
     def _get_model_view_transformation(self):
         if self.cached_transformation_matrix is None:
+            scale_factor = self.get_scale_factor()
+            # HiDPI: logical "device" (widget) pixels to screen pixels
             matrix = calculate_transformation_matrix(
-                self.scale, self.rotation,
+                self.scale / scale_factor, self.rotation,
                 self.translation_x, self.translation_y,
                 self.mirrored
             )
@@ -952,6 +967,15 @@ class CanvasRenderer (gtk.DrawingArea, DrawCursorMixin):
             self.transformation_updated()
         return self.cached_transformation_matrix
 
+    def _configure_event_cb(self, widget, event):
+        # The docs say to handle this on the toplevel to be notified of
+        # HiDPI scale_factor changes. Not sure yet whether this will
+        # work down at this level - I've no fancy HiDPI hardware to test
+        # with. For now, just try this.
+        logger.debug("configure-event received. Invalidating transform")
+        self._invalidate_cached_transform_matrix()
+        self.queue_draw()
+
     def is_translation_only(self):
         return self.rotation == 0.0 and self.scale == 1.0 and not self.mirrored
 
@@ -976,7 +1000,7 @@ class CanvasRenderer (gtk.DrawingArea, DrawCursorMixin):
         size = max(1, int(size))
 
         # Extract a square Cairo surface containing the area to sample
-        r = int(size/2)
+        r = int(size // 2)
         x = int(x) - r
         y = int(y) - r
         surf = self._new_image_surface_from_visible_area(
@@ -987,7 +1011,7 @@ class CanvasRenderer (gtk.DrawingArea, DrawCursorMixin):
 
         # Extract a pixbuf, then an average color.
         # surf.write_to_png("/tmp/grab.png")
-        pixbuf = gdk.pixbuf_get_from_surface(surf, 0, 0, size, size)
+        pixbuf = Gdk.pixbuf_get_from_surface(surf, 0, 0, size, size)
         color = lib.color.UIColor.new_from_pixbuf_average(pixbuf)
         return color
 
@@ -1065,12 +1089,9 @@ class CanvasRenderer (gtk.DrawingArea, DrawCursorMixin):
         # Don't render any partial views of the document if the widget
         # isn't sensitive to user input. If we don't do this, loading a
         # big doc might show each layer individually as it loads.
-        # Use a bit of alpha (over the default widget content) to make
-        # the transitions between sensitive & insensitive more apparent,
-        # and closer to the way Adwaita does things.
         if self._insensitive_state_content:
             cr.set_source_surface(self._insensitive_state_content, 0, 0)
-            cr.paint_with_alpha(0.8)
+            cr.paint()
             return True
 
         # Paint checkerboard if we won't be rendering an opaque background
@@ -1103,7 +1124,7 @@ class CanvasRenderer (gtk.DrawingArea, DrawCursorMixin):
         # Clear the pixbuf to be rendered with a random red,
         # to make it apparent if something is not being painted.
         if self.visualize_rendering:
-            surface.pixbuf.fill((int(random.random()*0xff) << 16)+0x00000000)
+            surface.pixbuf.fill(int(random.random() * 0xff) << 16)
 
         # Render to the pixbuf, then paint it.
         self._render_execute(
@@ -1163,16 +1184,16 @@ class CanvasRenderer (gtk.DrawingArea, DrawCursorMixin):
         """
 
         # Could this be an alternative?
-        #x0, y0, x1, y1 = cr.clip_extents()
-        #sparse = True
-        #clip_region = (x0, y0, x1-x0, y1-y0)
-        #return clip_region, sparse
+        # x0, y0, x1, y1 = cr.clip_extents()
+        # sparse = True
+        # clip_region = (x0, y0, x1-x0, y1-y0)
+        # return clip_region, sparse
 
         # Get the area which needs to be updated, in device coordinates, and
         # determine whether the render is "sparse", [TODO: define what this
         # means]
         x, y, w, h = device_bbox
-        cx, cy = x+w/2, y+h/2
+        cx, cy = x + w // 2, y + h // 2
 
         # As of 2012-07-08, Ubuntu Precise (LTS, unfortunately) and Debian
         # unstable(!) use python-cairo 1.8.8, which does not support
@@ -1193,7 +1214,7 @@ class CanvasRenderer (gtk.DrawingArea, DrawCursorMixin):
         # mingw-w64-i686-gobject-introspection-runtime or its x86_64
         # equivalent installed, also at version 1.44.
 
-        clip_exists, rect = gdk.cairo_get_clip_rectangle(cr)
+        clip_exists, rect = Gdk.cairo_get_clip_rectangle(cr)
         if clip_exists:
             # It's a wrapped cairo_rectangle_int_t, CairoRectangleInt
             # Convert to a better representation for our purposes,
@@ -1231,14 +1252,16 @@ class CanvasRenderer (gtk.DrawingArea, DrawCursorMixin):
         painting, or other activities that send partial updates.
 
         """
-        N = tiledsurface.N
+        n = tiledsurface.N
         if translation_only:
-            x, y = transformation.transform_point(tx*N, ty*N)
-            bbox = (int(x), int(y), N, N)
+            x, y = transformation.transform_point(tx * n, ty * n)
+            bbox = (int(x), int(y), n, n)
         else:
             corners = [
-                (tx*N, ty*N), ((tx+1)*N, ty*N),
-                (tx*N, (ty+1)*N), ((tx+1)*N, (ty+1)*N),
+                (tx * n, ty * n),
+                ((tx + 1) * n, ty * n),
+                (tx * n, (ty + 1) * n),
+                ((tx + 1) * n, (ty + 1) * n),
             ]
             corners = [
                 transformation.transform_point(x_, y_)
@@ -1275,9 +1298,9 @@ class CanvasRenderer (gtk.DrawingArea, DrawCursorMixin):
         # but for now, if the canvas is being panned around,
         # just render more simply.
         if self._hq_rendering:
-            mipmap_level = max(0, int(floor(log(1.0/self.scale, 2))))
+            mipmap_level = max(0, int(floor(log(1 / self.scale, 2))))
         else:
-            mipmap_level = max(0, int(ceil(log(1/self.scale, 2))))
+            mipmap_level = max(0, int(ceil(log(1 / self.scale, 2))))
 
         # OPTIMIZE: If we would render tile scanlines,
         # OPTIMIZE:  we could probably use the better one above...
@@ -1292,8 +1315,9 @@ class CanvasRenderer (gtk.DrawingArea, DrawCursorMixin):
         # calculate the final model bbox with all the clipping above
         x1, y1, x2, y2 = cr.clip_extents()
         if not self.is_translation_only():
-            # Looks like cairo needs one extra pixel rendered for interpolation at the border.
-            # If we don't do this, we get dark stripe artefacts when panning while zoomed.
+            # Looks like cairo needs one extra pixel rendered for
+            # interpolation at the border. If we don't do this, we get dark
+            # stripe artifacts when panning while zoomed.
             x1 -= 1
             y1 -= 1
             x2 += 1
@@ -1306,7 +1330,7 @@ class CanvasRenderer (gtk.DrawingArea, DrawCursorMixin):
         # factor 3 for ATI/Radeon Xorg driver (and hopefully others).
         # https://bugs.freedesktop.org/show_bug.cgi?id=28670
 
-        surface = pixbufsurface.Surface(x1, y1, x2-x1+1, y2-y1+1)
+        surface = pixbufsurface.Surface(x1, y1, x2 - x1 + 1, y2 - y1 + 1)
         return transformation, surface, sparse, mipmap_level, clip_rect
 
     def _render_execute(self, cr, transformation, surface, sparse,
@@ -1318,7 +1342,7 @@ class CanvasRenderer (gtk.DrawingArea, DrawCursorMixin):
         translation_only = self.is_translation_only()
 
         if self.visualize_rendering:
-            surface.pixbuf.fill((int(random.random()*0xff) << 16)+0x00000000)
+            surface.pixbuf.fill(int(random.random() * 0xff) << 16)
 
         fake_alpha_check_tile = None
         if not self._draw_real_alpha_checks:
@@ -1340,7 +1364,7 @@ class CanvasRenderer (gtk.DrawingArea, DrawCursorMixin):
 
         # Composite each stack of tiles in the exposed area
         # into the pixbufsurface.
-        self.doc._layers.render_into(
+        self.doc._layers.render(
             surface,
             tiles,
             mipmap_level,
@@ -1352,7 +1376,7 @@ class CanvasRenderer (gtk.DrawingArea, DrawCursorMixin):
         # Set the surface's underlying pixbuf as the source, then paint
         # it with Cairo. We don't care if it's pixelized at high zoom-in
         # levels: in fact, it'll look sharper and better.
-        gdk.cairo_set_source_pixbuf(
+        Gdk.cairo_set_source_pixbuf(
             cr, surface.pixbuf,
             round(surface.x), round(surface.y)
         )
@@ -1369,9 +1393,11 @@ class CanvasRenderer (gtk.DrawingArea, DrawCursorMixin):
         self.queue_draw()
 
         # This speeds things up nicely when scrolling is already
-        # fast, but produces temporary artefacts and an
+        # fast, but produces temporary artifacts and an
         # annoyingly non-constant framerate otherwise.
-        #self.window.scroll(int(-dx), int(-dy))
+        #
+        # self.window.scroll(int(-dx), int(-dy))
+        #
         # It might be worth it if it was done only once per
         # redraw, instead of once per motion event. Maybe try to
         # implement something like "queue_scroll" with priority
@@ -1383,7 +1409,7 @@ class CanvasRenderer (gtk.DrawingArea, DrawCursorMixin):
         """Return the center position in display coordinates.
         """
         alloc = self.get_allocation()
-        return alloc.width/2.0, alloc.height/2.0
+        return (alloc.width / 2.0, alloc.height / 2.0)
 
     def get_center_model_coords(self):
         """Return the center position in model coordinates.
@@ -1395,8 +1421,8 @@ class CanvasRenderer (gtk.DrawingArea, DrawCursorMixin):
         """Recentres the view onto the document's centre.
         """
         x, y, w, h = self.doc.get_effective_bbox()
-        cx = x+w/2.0
-        cy = y+h/2.0
+        cx = x + (w / 2.0)
+        cy = y + (h / 2.0)
         self.recenter_on_model_coords(cx, cy)
 
     def recenter_on_model_coords(self, cx, cy):
@@ -1411,7 +1437,7 @@ class CanvasRenderer (gtk.DrawingArea, DrawCursorMixin):
         self.translation_y += current_cy - cy
         self.queue_draw()
 
-    def defer_hq_rendering(self, t=1.0/8):
+    def defer_hq_rendering(self, t=1.0 / 8):
         """Use faster but lower-quality rendering for a brief period
 
         :param float t: The time to defer for, in seconds
@@ -1432,13 +1458,13 @@ class CanvasRenderer (gtk.DrawingArea, DrawCursorMixin):
 
         """
         if self._restore_hq_rendering_timeout_id:
-            glib.source_remove(self._restore_hq_rendering_timeout_id)
+            GLib.source_remove(self._restore_hq_rendering_timeout_id)
             self._restore_hq_rendering_timeout_id = None
         else:
             logger.debug("hq_rendering: deferring for %0.3fs...", t)
             self._hq_rendering = False
-        self._restore_hq_rendering_timeout_id = glib.timeout_add(
-            interval = int(t*1000),
+        self._restore_hq_rendering_timeout_id = GLib.timeout_add(
+            interval = int(t * 1000),
             function = self._resume_hq_rendering_timeout_cb,
         )
 
@@ -1472,12 +1498,12 @@ def _test():
     ctrlr = CanvasController(tdw)
     ctrlr.init_pointer_events()
     ctrlr.modes.default_mode_class = FreehandMode
-    win = gtk.Window()
+    win = Gtk.Window()
     win.set_title("tdw test")
-    win.connect("destroy", lambda *a: gtk.main_quit())
+    win.connect("destroy", lambda *a: Gtk.main_quit())
     win.add(tdw)
     win.show_all()
-    gtk.main()
+    Gtk.main()
     model.cleanup()
 
 
