@@ -12,15 +12,20 @@
 #include "common.hpp"
 #include "compositing.hpp"
 #include "blending.hpp"
+#include "fastapprox/fastpow.h"
 
 #include <mypaint-tiled-surface.h>
 
 #include <glib.h>
+#include <math.h>
 
 #define NPY_NO_DEPRECATED_API NPY_1_7_API_VERSION
 #define NO_IMPORT_ARRAY
 #include <numpy/arrayobject.h>
 
+#include <stdlib.h>
+#include <math.h>
+
 
 void
 tile_downscale_rgba16_c(const uint16_t *src, int src_strides, uint16_t *dst,
@@ -158,13 +163,13 @@ static void precalculate_dithering_noise_if_required()
   if (!have_noise) {
     // let's make some noise
     for (int i=0; i<dithering_noise_size; i++) {
-      // random number in range [0.03 .. 0.97] * (1<<15)
+      // random number in range [0.03 .. 0.2] * (1<<15)
       //
       // We could use the full range, but like this it is much easier
       // to guarantee 8bpc load-save roundtrips don't alter the
       // image. With the full range we would have to pay a lot
       // attention to rounding converting 8bpc to our internal format.
-      dithering_noise[i] = (rand() % (1<<15)) * 240/256 + (1<<15) * 8/256;
+      dithering_noise[i] = (rand() % (1<<15)) * 5/256 + (1<<15) * 2/256;
     }
     have_noise = true;
   }
@@ -173,12 +178,62 @@ static void precalculate_dithering_noise_if_required()
 // Used for saving layers (transparent PNG), and for display when there
 // can be transparent areas in the output.
 
+static inline void
+tile_convert_rgba16_to_rgba8_const_c (const uint16_t* const src,
+				      const int src_strides,
+				      const uint8_t* dst,
+				      const int dst_strides)
+{
+  precalculate_dithering_noise_if_required();
+
+  for (int y=0; y<MYPAINT_TILE_SIZE; y++) {
+    int noise_idx = y*MYPAINT_TILE_SIZE*4;
+    const uint16_t *src_p = (uint16_t*)((char *)src + y*src_strides);
+    uint8_t *dst_p = (uint8_t*)((char *)dst + y*dst_strides);
+    for (int x=0; x<MYPAINT_TILE_SIZE; x++) {
+      uint32_t r, g, b, a;
+      r = *src_p++;
+      g = *src_p++;
+      b = *src_p++;
+      a = *src_p++;
+      // un-premultiply alpha (with rounding)
+      if (a != 0) {
+        const uint32_t rnd_a = a/2;
+        r = ((r << 15) + rnd_a) / a;
+        g = ((g << 15) + rnd_a) / a;
+        b = ((b << 15) + rnd_a) / a;
+      } else {
+        r = g = b = 0;
+      }
+      const uint32_t add_r = dithering_noise[noise_idx+0];
+      const uint32_t add_g = add_r; // hm... do not produce too much color noise
+      const uint32_t add_b = add_r;
+      const uint32_t add_a = dithering_noise[noise_idx+1];
+      noise_idx += 4;
+
+
+      *dst_p++ = (r * 255 + add_r) / (1<<15);
+      *dst_p++ = (g * 255 + add_g) / (1<<15);
+      *dst_p++ = (b * 255 + add_b) / (1<<15);
+      *dst_p++ = (a * 255 + add_a) / (1<<15);
+    }
+    src_p += src_strides;
+    dst_p += dst_strides;
+  }
+}
+
+
 static inline void
 tile_convert_rgba16_to_rgba8_c (const uint16_t* const src,
                                 const int src_strides,
                                 const uint8_t* dst,
-                                const int dst_strides)
+                                const int dst_strides,
+                                const float EOTF)
 {
+  if (EOTF == 1.0) {
+    tile_convert_rgba16_to_rgba8_const_c(src, src_strides, dst, dst_strides);
+    return;
+  }
   precalculate_dithering_noise_if_required();
 
   for (int y=0; y<MYPAINT_TILE_SIZE; y++) {
@@ -245,9 +300,9 @@ tile_convert_rgba16_to_rgba8_c (const uint16_t* const src,
 
       // Variant C) but with precalculated noise (much faster)
       //
-      const uint32_t add_r = dithering_noise[noise_idx+0];
-      const uint32_t add_g = add_r; // hm... do not produce too much color noise
-      const uint32_t add_b = add_r;
+      const float add_r = (float)dithering_noise[noise_idx+0] / (1<<30);
+      //const uint32_t add_g = add_r; // hm... do not produce too much color noise
+      //const uint32_t add_b = add_r;
       const uint32_t add_a = dithering_noise[noise_idx+1];
       noise_idx += 4;
 
@@ -257,10 +312,10 @@ tile_convert_rgba16_to_rgba8_c (const uint16_t* const src,
       assert(noise_idx <= dithering_noise_size);
 #endif
 
-      *dst_p++ = (r * 255 + add_r) / (1<<15);
-      *dst_p++ = (g * 255 + add_g) / (1<<15);
-      *dst_p++ = (b * 255 + add_b) / (1<<15);
-      *dst_p++ = (a * 255 + add_a) / (1<<15);
+      *dst_p++ = uint8_t(fastpow((float)r / (1<<15) + add_r, 1.0/EOTF) * 255);
+      *dst_p++ = uint8_t(fastpow((float)g / (1<<15) + add_r, 1.0/EOTF) * 255);
+      *dst_p++ = uint8_t(fastpow((float)b / (1<<15) + add_r, 1.0/EOTF) * 255);
+      *dst_p++ = ((a * 255 + add_a) / (1<<15));
     }
     src_p += src_strides;
     dst_p += dst_strides;
@@ -271,7 +326,7 @@ tile_convert_rgba16_to_rgba8_c (const uint16_t* const src,
 
 void
 tile_convert_rgba16_to_rgba8 (PyObject *src,
-                              PyObject *dst)
+                              PyObject *dst, const float EOTF)
 {
   PyArrayObject* src_arr = ((PyArrayObject*)src);
   PyArrayObject* dst_arr = ((PyArrayObject*)dst);
@@ -299,14 +354,15 @@ tile_convert_rgba16_to_rgba8 (PyObject *src,
   tile_convert_rgba16_to_rgba8_c((uint16_t*)PyArray_DATA(src_arr),
                                  PyArray_STRIDES(src_arr)[0],
                                  (uint8_t*)PyArray_DATA(dst_arr),
-                                 PyArray_STRIDES(dst_arr)[0]);
+                                 PyArray_STRIDES(dst_arr)[0],
+                                 EOTF);
 }
 
 static inline void
-tile_convert_rgbu16_to_rgbu8_c(const uint16_t* const src,
-                               const int src_strides,
-                               const uint8_t* dst,
-                               const int dst_strides)
+tile_convert_rgbu16_to_rgbu8_const_c(const uint16_t *const src,
+                                     const int src_strides,
+				     const uint8_t *dst,
+                                     const int dst_strides)
 {
   precalculate_dithering_noise_if_required();
 
@@ -319,6 +375,40 @@ tile_convert_rgbu16_to_rgbu8_c(const uint16_t* const src,
       r = *src_p++;
       g = *src_p++;
       b = *src_p++;
+      src_p++;
+      const uint32_t add = dithering_noise[noise_idx++];
+      *dst_p++ = (r * 255 + add) / (1<<15);
+      *dst_p++ = (g * 255 + add) / (1<<15);
+      *dst_p++ = (b * 255 + add) / (1<<15);
+      *dst_p++ = 255;
+    }
+    src_p += src_strides;
+    dst_p += dst_strides;
+  }
+}
+
+static inline void
+tile_convert_rgbu16_to_rgbu8_c(const uint16_t* const src,
+                               const int src_strides,
+                               const uint8_t* dst,
+                               const int dst_strides,
+                               const float EOTF)
+{
+  if (EOTF == 1.0) {
+    tile_convert_rgbu16_to_rgbu8_const_c(src, src_strides, dst, dst_strides);
+    return;
+  }
+  precalculate_dithering_noise_if_required();
+
+  for (int y=0; y<MYPAINT_TILE_SIZE; y++) {
+    int noise_idx = y*MYPAINT_TILE_SIZE*4;
+    const uint16_t *src_p = (uint16_t*)((char *)src + y*src_strides);
+    uint8_t *dst_p = (uint8_t*)((char *)dst + y*dst_strides);
+    for (int x=0; x<MYPAINT_TILE_SIZE; x++) {
+      float r, g, b;
+      r = ((float)*src_p++ / (1<<15));
+      g = ((float)*src_p++ / (1<<15));
+      b = ((float)*src_p++ / (1<<15));
       src_p++; // alpha unused
 #ifdef HEAVY_DEBUG
       assert(r<=(1<<15));
@@ -331,11 +421,11 @@ tile_convert_rgbu16_to_rgbu8_c(const uint16_t* const src,
       const uint32_t add = (1<<15)/2;
       */
       // dithering
-      const uint32_t add = dithering_noise[noise_idx++];
+      const float add = (float)dithering_noise[noise_idx++] / (1<<30);
 
-      *dst_p++ = (r * 255 + add) / (1<<15);
-      *dst_p++ = (g * 255 + add) / (1<<15);
-      *dst_p++ = (b * 255 + add) / (1<<15);
+      *dst_p++ = (fastpow(r + add, 1.0/EOTF) ) * 255 + 0.5;
+      *dst_p++ = (fastpow(g + add, 1.0/EOTF) ) * 255 + 0.5;
+      *dst_p++ = (fastpow(b + add, 1.0/EOTF) ) * 255 + 0.5;
       *dst_p++ = 255;
     }
 #ifdef HEAVY_DEBUG
@@ -347,7 +437,7 @@ tile_convert_rgbu16_to_rgbu8_c(const uint16_t* const src,
 }
 
 
-void tile_convert_rgbu16_to_rgbu8(PyObject * src, PyObject * dst) {
+void tile_convert_rgbu16_to_rgbu8(PyObject * src, PyObject * dst, const float EOTF) {
   PyArrayObject* src_arr = ((PyArrayObject*)src);
   PyArrayObject* dst_arr = ((PyArrayObject*)dst);
 
@@ -372,12 +462,46 @@ void tile_convert_rgbu16_to_rgbu8(PyObject * src, PyObject * dst) {
 #endif
 
   tile_convert_rgbu16_to_rgbu8_c((uint16_t*)PyArray_DATA(src_arr), PyArray_STRIDES(src_arr)[0],
-                                 (uint8_t*)PyArray_DATA(dst_arr), PyArray_STRIDES(dst_arr)[0]);
+                                 (uint8_t*)PyArray_DATA(dst_arr), PyArray_STRIDES(dst_arr)[0],
+                                  EOTF);
+}
+
+void tile_convert_rgba8_to_rgba16_const(PyObject * src, PyObject * dst) {
+  PyArrayObject* src_arr = ((PyArrayObject*)src);
+  PyArrayObject* dst_arr = ((PyArrayObject*)dst);
+  for (int y=0; y<MYPAINT_TILE_SIZE; y++) {
+    uint8_t  * src_p = (uint8_t*)((char *)PyArray_DATA(src_arr) + y*PyArray_STRIDES(src_arr)[0]);
+    uint16_t * dst_p = (uint16_t*)((char *)PyArray_DATA(dst_arr) + y*PyArray_STRIDES(dst_arr)[0]);
+    for (int x=0; x<MYPAINT_TILE_SIZE; x++) {
+      uint32_t r, g, b, a;
+      r = *src_p++;
+      g = *src_p++;
+      b = *src_p++;
+      a = *src_p++;
+
+      // convert to fixed point (with rounding)
+      r = (r * (1<<15) + 255/2) / 255;
+      g = (g * (1<<15) + 255/2) / 255;
+      b = (b * (1<<15) + 255/2) / 255;
+      a = (a * (1<<15) + 255/2) / 255;
+
+      // premultiply alpha (with rounding), save back
+      *dst_p++ = (r * a + (1<<15)/2) / (1<<15);
+      *dst_p++ = (g * a + (1<<15)/2) / (1<<15);
+      *dst_p++ = (b * a + (1<<15)/2) / (1<<15);
+      *dst_p++ = a;
+    }
+  }
 }
 
 
 // used mainly for loading layers (transparent PNG)
-void tile_convert_rgba8_to_rgba16(PyObject * src, PyObject * dst) {
+void tile_convert_rgba8_to_rgba16(PyObject * src, PyObject * dst, const float EOTF) {
+
+  if (EOTF == 1.0) {
+    tile_convert_rgba8_to_rgba16_const(src, dst);
+    return;
+  }
   PyArrayObject* src_arr = ((PyArrayObject*)src);
   PyArrayObject* dst_arr = ((PyArrayObject*)dst);
 
@@ -412,9 +536,9 @@ void tile_convert_rgba8_to_rgba16(PyObject * src, PyObject * dst) {
       a = *src_p++;
 
       // convert to fixed point (with rounding)
-      r = (r * (1<<15) + 255/2) / 255;
-      g = (g * (1<<15) + 255/2) / 255;
-      b = (b * (1<<15) + 255/2) / 255;
+      r = uint32_t(fastpow((float)r/255.0, EOTF) * (1<<15) + 0.5);
+      g = uint32_t(fastpow((float)g/255.0, EOTF) * (1<<15) + 0.5);
+      b = uint32_t(fastpow((float)b/255.0, EOTF) * (1<<15) + 0.5);
       a = (a * (1<<15) + 255/2) / 255;
 
       // premultiply alpha (with rounding), save back
@@ -682,7 +806,8 @@ static const TileDataCombineOp * combine_mode_info[NumCombineModes] =
     new TileDataCombine<BlendNormal, CompositeDestinationIn>("svg:dst-in"),
     new TileDataCombine<BlendNormal, CompositeDestinationOut>("svg:dst-out"),
     new TileDataCombine<BlendNormal, CompositeSourceAtop>("svg:src-atop"),
-    new TileDataCombine<BlendNormal, CompositeDestinationAtop>("svg:dst-atop")
+    new TileDataCombine<BlendNormal, CompositeDestinationAtop>("svg:dst-atop"),
+    new TileDataCombine<BlendNormal, CompositeSpectralWGM>("mypaint:spectral-wgm")
 };
 
 
